﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using System.IO;
using Microsoft.VisualStudio.Shell.Interop;
using Microsoft.VisualStudio;
using Microsoft.VisualStudio.Shell;
using EnvDTE;
using System.Runtime.InteropServices;
using Microsoft.VisualBasic;
using System.Collections;
using Microsoft.VisualStudio.OLE.Interop;
using System.Diagnostics;
using System.Reflection;
using VSLangProj;
using VSLangProj80;
using EnvDTE80;

namespace Itcast.JSMin.Helpers
{
    public class IDEUtils
    {
        public const string CUSTOMTOOL = "CustomTool";
        //默认名字空间
        public const string DEFAULTNAMESPACE = "DefaultNameSpace";

        public static void SetCustomTool(ProjectItem projectItem, string value)
        {
            projectItem.Properties.Item(CUSTOMTOOL).Value = value;
        }

        public static string GetCustomTool(ProjectItem projectItem)
        {
            Property prop = projectItem.Properties.Item(CUSTOMTOOL);
            if (prop == null)
            {
                return null;
            }
            return (string)prop.Value;
        }

        private static IVsOutputWindow OutputWindowInstance
        {
            get
            {
                return Package.GetGlobalService(typeof(SVsOutputWindow)) as IVsOutputWindow;
            }
        }

        public static void WriteToOutput(string message)
        {
            Guid OutputPaneGuid = VSConstants.GUID_OutWindowGeneralPane;
            IVsOutputWindowPane OutputPane = null;
            int vsResult = OutputWindowInstance.GetPane(ref OutputPaneGuid, out OutputPane);
            if ((vsResult != VSConstants.S_OK) || (OutputPane == null))
                return;
            OutputPane.OutputStringThreadSafe(message);
        }

        public static IVsStatusbar VsStatusbar
        {
            get
            {
                return Package.GetGlobalService(typeof(SVsStatusbar)) as IVsStatusbar;
            }
        }

        public static void SetStatusbarText(string text)
        {
            VsStatusbar.SetText(text);
        }

        public static void LogException(Exception ex)
        {
            StringBuilder sb = new StringBuilder();
            sb.AppendLine(DateTime.Now.ToString());
            sb.AppendLine(ex.Message);
            sb.AppendLine(ex.StackTrace);
            sb.AppendLine("********************************************************");
            Debug.WriteLine(sb.ToString());
        }

        /// <summary>
        /// 得到全局VsUIShell
        /// </summary>
        /// <returns></returns>
        public static IVsUIShell GetIVsUIShell()
        {
            IVsUIShell uiShell = (IVsUIShell)Package.GetGlobalService(typeof(SVsUIShell));
            if (uiShell == null)
            {
                throw new ApplicationException("No IVsActivityLog of GetGlobalService was found!");
            }
            return uiShell;
        }

        /// <summary>
        /// 得到全局的DTE
        /// </summary>
        /// <returns></returns>
        public static DTE GetDTE()
        {
            DTE dte = (DTE)Package.GetGlobalService(typeof(DTE));
            if (dte == null)
            {
                throw new ApplicationException("No DTE of GetGlobalService was found!");
            }
            return dte;
        }

        /// <summary>
        /// 得到全局的Solution
        /// </summary>
        /// <returns></returns>
        public static IVsSolution GetIVsSolution()
        {
            IVsSolution solution = (IVsSolution)Package.GetGlobalService(typeof(SVsSolution));
            return solution;
        }

        /// <summary>
        /// 得到当前打开的文档所在的工程
        /// </summary>
        /// <returns></returns>
        public static EnvDTE.Project GetActiveDocumentProject()
        {
            EnvDTE.DTE dte = IDEUtils.GetDTE();
            if (dte.ActiveDocument == null)
            {
                throw new Exception("当前没有打开的激活文档！");
            }
            return dte.ActiveDocument.ProjectItem.ContainingProject;
        }

        /// <summary>
        /// 得到当前打开的文档所在的工程
        /// </summary>
        /// <returns></returns>
        public static ProjectItem GetActiveDocumentProjectItem()
        {
            EnvDTE.DTE dte = IDEUtils.GetDTE();
            return dte.ActiveDocument.ProjectItem;
        }

        /// <summary>
        /// 得到当前选中的元素所在的工程
        /// 当前选中的工程必须是一个且仅一个
        /// </summary>
        /// <returns></returns>
        public static EnvDTE.Project GetActiveSingleProject()
        {
            EnvDTE.DTE dte = IDEUtils.GetDTE();
            object[] projects = (object[])dte.ActiveSolutionProjects;
            if (projects.Length != 1)
            {
                throw new SystemException("projects.Length != 1");
            }
            EnvDTE.Project project = (EnvDTE.Project)projects[0];
            return project;
        }

        /// <summary>
        /// 得到当前选中的元素所在的工程的IVsHierarchy
        /// 当前选中的工程必须是一个且仅一个
        /// </summary>
        /// <returns></returns>
        public static IVsHierarchy GetActiveSingleProjectVsHierarchy()
        {
            EnvDTE.Project project = IDEUtils.GetActiveSingleProject();
            string uniqueName = project.UniqueName;
            IVsSolution solution = IDEUtils.GetIVsSolution();
            IVsHierarchy hierarchy;
            ErrorHandler.ThrowOnFailure(solution.GetProjectOfUniqueName(uniqueName, out hierarchy));
            return hierarchy;
        }

        /// <summary>
        /// 得到当前选中的元素所在的工程的IVsBuildPropertyStorage
        /// 当前选中的工程必须是一个且仅一个
        /// </summary>
        /// <returns></returns>
        public static IVsBuildPropertyStorage GetActiveSingleProjectVsBuildPropertyStorage()
        {
            IVsHierarchy hierarchy = GetActiveSingleProjectVsHierarchy();
            return hierarchy as IVsBuildPropertyStorage;
        }
        /// <summary>
        /// 得到当前选中的对象
        /// 当前选中的工程必须是一个且仅一个
        /// </summary>
        /// <returns></returns>
        public static SelectedItem GetDTESingleSelectedItem()
        {
            SelectedItems items = IDEUtils.GetDTE().SelectedItems;
            if (items.Count != 1)
            {
                throw new InvalidOperationException("items.Count != 1, 请选择当个交易进行部署");
            }
            return items.Item(1);
        }

        /// <summary>
        /// 设置projectItem的name属性为值value
        /// </summary>
        /// <param name="projectItem"></param>
        /// <param name="name"></param>
        /// <param name="value"></param>
        public static void SetProjectItemValue(ProjectItem projectItem, string name, string value)
        {
            IVsHierarchy hierarchy = GetActiveSingleProjectVsHierarchy();
            IVsBuildPropertyStorage storage = GetActiveSingleProjectVsBuildPropertyStorage();

            uint itemId;
            string fullPath = GetFullName(projectItem);
            ErrorHandler.ThrowOnFailure(hierarchy.ParseCanonicalName(fullPath, out itemId));
            ErrorHandler.ThrowOnFailure(storage.SetItemAttribute(itemId, name, value));
        }

        /// <summary>
        /// 在projectItem上运行Custom Tool
        /// </summary>
        /// <param name="projectItem"></param>
        public static void RunCustomTool(ProjectItem projectItem)
        {
            Debug.WriteLine("RunCustomTool for item:" + projectItem.Name);

            try
            {
                VSProjectItem item = projectItem.Object as VSProjectItem;
                Debug.Assert(item != null);
                item.RunCustomTool();
            }
            catch(Exception ex)
            {
                IDEUtils.LogException(ex);
            }
            Debug.WriteLine("RunCustomTool finished for item:" + projectItem.Name);
        }

        /// <summary>
        /// 对在ProjectItems中的所有扩展名为符合要求的文件运行ProjectItem
        /// </summary>
        /// <param name="Items"></param>
        /// <param name="Exts"></param>
        public static void RunCustomTool(ProjectItems Items, String[] Exts)
        {
            if (Items == null || Items.Count < 1 || Exts == null || Exts.Length < 1) return;

            foreach (ProjectItem item in Items)
            {
                if (item.Kind == EnvDTE.Constants.vsProjectItemKindPhysicalFile)
                { // 如果是文件
                    foreach (String ext in Exts)
                        if (item.Name.EndsWith(ext)) RunCustomTool(item);
                }
                else if (item.Kind == EnvDTE.Constants.vsProjectItemKindPhysicalFolder)
                    RunCustomTool(item.ProjectItems, Exts);
                else
                    Debug.WriteLine(item.Name);
            }
        }

        /// <summary>
        /// 取得projectItem的name属性值
        /// </summary>
        /// <param name="projectItem"></param>
        /// <param name="name"></param>
        /// <returns></returns>
        public static string GetProjectItemValue(ProjectItem projectItem, string name)
        {
            IVsHierarchy hierarchy = GetActiveSingleProjectVsHierarchy();
            IVsBuildPropertyStorage storage = GetActiveSingleProjectVsBuildPropertyStorage();

            uint itemId;
            string fullPath = GetFullName(projectItem);
            ErrorHandler.ThrowOnFailure(hierarchy.ParseCanonicalName(fullPath, out itemId));

            string value;
            ErrorHandler.ThrowOnFailure(storage.GetItemAttribute(itemId, name, out value));
            return value;
        }        

        /// <summary>
        /// 取得一个ProjectItem的全路径
        /// </summary>
        /// <param name="projectItem"></param>
        /// <returns></returns>
        public static string GetFullName(ProjectItem projectItem)
        {
            return (string)projectItem.Properties.Item("FullPath").Value;
        }

        /// <summary>
        /// 取得一个ProjectItem的默认名字空间
        /// </summary>
        /// <param name="projectItem"></param>
        /// <returns></returns>
        public static string GetDefaultNameSpace(ProjectItem projectItem)
        {
            return (string)projectItem.Properties.Item(DEFAULTNAMESPACE).Value;
        } 

        /// <summary>
        /// 尝试用设计器打开文件
        /// </summary>
        /// <param name="filename"></param>
        public static Window OpenFileInDesigner(string filename)
        {
            string kind = EnvDTE.Constants.vsViewKindDesigner;
            Window window = GetDTE().ItemOperations.OpenFile(filename, kind);

            //激活窗口，将其放到最前端,因为如果窗口已经打开，并且不是当前窗口
            //则不能很清晰的看到打开这个过程
            window.Activate();
            return window;
        }

        /// <summary>
        /// 尝试用代码编辑器打开文件
        /// </summary>
        /// <param name="filename"></param>
        public static Window OpenFileAsCode(string filename)
        {
            string kind = EnvDTE.Constants.vsViewKindCode;
            Window window = GetDTE().ItemOperations.OpenFile(filename, kind);

            //激活窗口，将其放到最前端,因为如果窗口已经打开，并且不是当前窗口
            //则不能很清晰的看到打开这个过程
            window.Activate();
            return window;
        }
    }
}
